@@ -188,7 +188,7 @@ def compare_feature(old_f, new_f):
                         % (repr(old_f.qualifiers[key]),
                            repr(new_f.qualifiers[key]))
             else:
-                assert False, "Problem with feature's '%s' qualifier" & key
+                assert False, "Problem with feature's '%s' qualifier" % key
         else:
             # Should both be lists of strings...
             assert old_f.qualifiers[key] == new_f.qualifiers[key], \
@@ -217,7 +217,7 @@ def compare_sequence(old, new):
         indices = list(range(-ln, ln))
     else:
         # A selection of end cases, and the mid point
-        indices = [-ln, -ln+1, -(ln//2), -1, 0, 1, ln//2, ln-2, ln-1]
+        indices = [-ln, -ln + 1, -(ln // 2), -1, 0, 1, ln // 2, ln - 2, ln - 1]
 
     # Test element access,
     for i in indices:
@@ -291,12 +291,13 @@ def compare_record(old, new):
     # TODO - address these, see Bug 2681?
     new_keys = set(new.annotations).difference(old.annotations)
     new_keys = new_keys.difference(['cross_references', 'date',
-                                    'data_file_division', 'ncbi_taxid', 'gi'])
+                                    'data_file_division', 'ncbi_taxid', 
+                                    'gi'])
     assert not new_keys, "Unexpected new annotation keys: %s" \
            % ", ".join(new_keys)
     missing_keys = set(old.annotations).difference(new.annotations)
     missing_keys = missing_keys.difference(['ncbi_taxid',  # Can't store chimeras
-                                            ])
+                                            'structured_comment'])
     assert not missing_keys, "Unexpectedly missing annotation keys: %s" \
            % ", ".join(missing_keys)
 
