@@ -66,7 +66,7 @@ def compare_record(old, new, truncate=None):
     if old.id != new.id:
         raise ValueError("'%s' vs '%s' " % (old.id, new.id))
     if old.description != new.description \
-    and (old.id+" "+old.description).strip() != new.description:
+    and (old.id + " " + old.description).strip() != new.description:
         raise ValueError("'%s' vs '%s' " % (old.description, new.description))
     if len(old.seq) != len(new.seq):
         raise ValueError("%i vs %i" % (len(old.seq), len(new.seq)))
@@ -212,7 +212,7 @@ tests = [("del", 3, 5),
          ("null", 0, 5)]
 for base_name, good_count, full_count in tests:
     def funct(name, c1, c2):
-        f = lambda x: x.check_qual_char("Quality/error_qual_%s.fastq"%name, c1, c2)
+        f = lambda x: x.check_qual_char("Quality/error_qual_%s.fastq" % name, c1, c2)
         f.__doc__ = "Reject FASTQ with %s in quality" % name.replace("_", " ")
         return f
     setattr(TestFastqErrors, "test_qual_%s" % (base_name),
@@ -232,9 +232,9 @@ class TestReferenceSffConversions(unittest.TestCase):
         for old, new in zip(wanted, converted):
             self.assertEqual(old.id, new.id)
             self.assertEqual(old.name, new.name)
-            if format!="qual":
+            if format != "qual":
                 self.assertEqual(str(old.seq), str(new.seq))
-            elif format!="fasta":
+            elif format != "fasta":
                 self.assertEqual(old.letter_annotations["phred_quality"],
                                  new.letter_annotations["phred_quality"])
 
@@ -289,16 +289,15 @@ class TestReferenceFastqConversions(unittest.TestCase):
                 if out_variant != "sanger":
                     # Ignore data loss warnings from max qualities
                     warnings.simplefilter("ignore", BiopythonWarning)
-                    warnings.simplefilter("ignore", UserWarning)
                 # Check matches using convert...
                 handle = StringIO()
-                SeqIO.convert(in_filename, "fastq-"+in_variant,
-                              handle, "fastq-"+out_variant)
+                SeqIO.convert(in_filename, "fastq-" + in_variant,
+                              handle, "fastq-" + out_variant)
                 self.assertEqual(expected, handle.getvalue())
                 # Check matches using parse/write
                 handle = StringIO()
-                SeqIO.write(SeqIO.parse(in_filename, "fastq-"+in_variant),
-                            handle, "fastq-"+out_variant)
+                SeqIO.write(SeqIO.parse(in_filename, "fastq-" + in_variant),
+                            handle, "fastq-" + out_variant)
                 self.assertEqual(expected, handle.getvalue())
 
 
@@ -329,7 +328,7 @@ class TestQual(unittest.TestCase):
         """Check FASTQ parsing matches FASTA+QUAL parsing"""
         with open("Quality/example.fasta") as f:
             with open("Quality/example.qual") as q:
-                records1 = list(QualityIO.PairedFastaQualIterator(f, q ))
+                records1 = list(QualityIO.PairedFastaQualIterator(f, q))
         records2 = list(SeqIO.parse("Quality/example.fastq", "fastq"))
         self.assertTrue(compare_records(records1, records2))
 
@@ -401,7 +400,7 @@ class TestReadWrite(unittest.TestCase):
     def test_fastq_2000(self):
         """Read and write back simple example with upper case 2000bp read"""
         data = "@%s\n%s\n+\n%s\n" \
-               % ("id descr goes here", "ACGT"*500, "!@a~"*500)
+               % ("id descr goes here", "ACGT" * 500, "!@a~" * 500)
         handle = StringIO()
         self.assertEqual(1, SeqIO.write(SeqIO.parse(StringIO(data), "fastq"), handle, "fastq"))
         self.assertEqual(data, handle.getvalue())
@@ -409,7 +408,7 @@ class TestReadWrite(unittest.TestCase):
     def test_fastq_1000(self):
         """Read and write back simple example with mixed case 1000bp read"""
         data = "@%s\n%s\n+\n%s\n" \
-               % ("id descr goes here", "ACGTNncgta"*100, "abcd!!efgh"*100)
+               % ("id descr goes here", "ACGTNncgta" * 100, "abcd!!efgh" * 100)
         handle = StringIO()
         self.assertEqual(1, SeqIO.write(SeqIO.parse(StringIO(data), "fastq"), handle, "fastq"))
         self.assertEqual(data, handle.getvalue())
@@ -420,7 +419,7 @@ class TestReadWrite(unittest.TestCase):
         data = "@%s\n%s\n+\n%s\n" \
                % ("id descr goes here",
                   ambiguous_dna_letters.upper(),
-                  "".join(chr(33+q) for q in range(len(ambiguous_dna_letters))))
+                  "".join(chr(33 + q) for q in range(len(ambiguous_dna_letters))))
         handle = StringIO()
         self.assertEqual(1, SeqIO.write(SeqIO.parse(StringIO(data), "fastq"), handle, "fastq"))
         self.assertEqual(data, handle.getvalue())
@@ -428,7 +427,7 @@ class TestReadWrite(unittest.TestCase):
         data = "@%s\n%s\n+\n%s\n" \
                % ("id descr goes here",
                   ambiguous_dna_letters.lower(),
-                  "".join(chr(33+q) for q in range(len(ambiguous_dna_letters))))
+                  "".join(chr(33 + q) for q in range(len(ambiguous_dna_letters))))
         handle = StringIO()
         self.assertEqual(1, SeqIO.write(SeqIO.parse(StringIO(data), "fastq"), handle, "fastq"))
         self.assertEqual(data, handle.getvalue())
@@ -439,7 +438,7 @@ class TestReadWrite(unittest.TestCase):
         data = "@%s\n%s\n+\n%s\n" \
                % ("id descr goes here",
                   ambiguous_rna_letters.upper(),
-                  "".join(chr(33+q) for q in range(len(ambiguous_rna_letters))))
+                  "".join(chr(33 + q) for q in range(len(ambiguous_rna_letters))))
         handle = StringIO()
         self.assertEqual(1, SeqIO.write(SeqIO.parse(StringIO(data), "fastq"), handle, "fastq"))
         self.assertEqual(data, handle.getvalue())
@@ -447,7 +446,7 @@ class TestReadWrite(unittest.TestCase):
         data = "@%s\n%s\n+\n%s\n" \
                % ("id descr goes here",
                   ambiguous_rna_letters.lower(),
-                  "".join(chr(33+q) for q in range(len(ambiguous_rna_letters))))
+                  "".join(chr(33 + q) for q in range(len(ambiguous_rna_letters))))
         handle = StringIO()
         self.assertEqual(1, SeqIO.write(SeqIO.parse(StringIO(data), "fastq"), handle, "fastq"))
         self.assertEqual(data, handle.getvalue())
@@ -457,21 +456,21 @@ class TestWriteRead(unittest.TestCase):
     """Test can write and read back files."""
     def test_generated(self):
         """Write and read back odd SeqRecord objects"""
-        record1 = SeqRecord(Seq("ACGT"*500, generic_dna),  id="Test", description="Long "*500,
-                           letter_annotations={"phred_quality": [40, 30, 20, 10]*500})
-        record2 = SeqRecord(MutableSeq("NGGC"*1000),  id="Mut", description="very "*1000+"long",
-                           letter_annotations={"phred_quality": [0, 5, 5, 10]*1000})
-        record3 = SeqRecord(UnknownSeq(2000, character="N"),  id="Unk", description="l"+("o"*1000)+"ng",
-                           letter_annotations={"phred_quality": [0, 1]*1000})
-        record4 = SeqRecord(Seq("ACGT"*500),  id="no_descr", description="", name="",
-                           letter_annotations={"phred_quality": [40, 50, 60, 62]*500})
-        record5 = SeqRecord(Seq("", generic_dna),  id="empty_p", description="(could have been trimmed lots)",
+        record1 = SeqRecord(Seq("ACGT" * 500, generic_dna), id="Test", description="Long " * 500,
+                           letter_annotations={"phred_quality": [40, 30, 20, 10] * 500})
+        record2 = SeqRecord(MutableSeq("NGGC" * 1000), id="Mut", description="very " * 1000 + "long",
+                           letter_annotations={"phred_quality": [0, 5, 5, 10] * 1000})
+        record3 = SeqRecord(UnknownSeq(2000, character="N"), id="Unk", description="l" + ("o" * 1000) + "ng",
+                           letter_annotations={"phred_quality": [0, 1] * 1000})
+        record4 = SeqRecord(Seq("ACGT" * 500), id="no_descr", description="", name="",
+                           letter_annotations={"phred_quality": [40, 50, 60, 62] * 500})
+        record5 = SeqRecord(Seq("", generic_dna), id="empty_p", description="(could have been trimmed lots)",
                            letter_annotations={"phred_quality": []})
-        record6 = SeqRecord(Seq(""),  id="empty_s", description="(could have been trimmed lots)",
+        record6 = SeqRecord(Seq(""), id="empty_s", description="(could have been trimmed lots)",
                            letter_annotations={"solexa_quality": []})
-        record7 = SeqRecord(Seq("ACNN"*500),  id="Test_Sol", description="Long "*500,
-                           letter_annotations={"solexa_quality": [40, 30, 0, -5]*500})
-        record8 = SeqRecord(Seq("ACGT"),  id="HighQual", description="With very large qualities that even Sanger FASTQ can't hold!",
+        record7 = SeqRecord(Seq("ACNN" * 500), id="Test_Sol", description="Long " * 500,
+                           letter_annotations={"solexa_quality": [40, 30, 0, -5] * 500})
+        record8 = SeqRecord(Seq("ACGT"), id="HighQual", description="With very large qualities that even Sanger FASTQ can't hold!",
                            letter_annotations={"solexa_quality": [0, 10, 100, 1000]})
         # TODO - Record with no identifier?
         records = [record1, record2, record3, record4, record5, record6, record7, record8]
@@ -500,8 +499,11 @@ class TestWriteRead(unittest.TestCase):
         """Write and read back sanger_93.fastq"""
         self.check(os.path.join("Quality", "sanger_93.fastq"), "fastq",
                    ["fastq", "fastq-sanger", "fasta", "qual", "phd"])
-        self.check(os.path.join("Quality", "sanger_93.fastq"), "fastq",
-                   ["fastq-solexa", "fastq-illumina"])
+        with warnings.catch_warnings():
+            # TODO - Have a Biopython defined "DataLossWarning?"
+            warnings.simplefilter('ignore', BiopythonWarning)
+            self.check(os.path.join("Quality", "sanger_93.fastq"), "fastq",
+                       ["fastq-solexa", "fastq-illumina"])
 
     def test_sanger_faked(self):
         """Write and read back sanger_faked.fastq"""
@@ -646,8 +648,8 @@ class MappingTests(unittest.TestCase):
         # The point of this test is the writing code doesn't actually use the
         # solexa_quality_from_phred function directly. For speed it uses a
         # cached dictionary of the mappings.
-        seq = "N"*94
-        qual = "".join(chr(33+q) for q in range(0, 94))
+        seq = "N" * 94
+        qual = "".join(chr(33 + q) for q in range(0, 94))
         expected_sol = [min(62, int(round(QualityIO.solexa_quality_from_phred(q))))
                         for q in range(0, 94)]
         in_handle = StringIO("@Test\n%s\n+\n%s" % (seq, qual))
@@ -668,8 +670,8 @@ class MappingTests(unittest.TestCase):
         # The point of this test is the writing code doesn't actually use the
         # solexa_quality_from_phred function directly. For speed it uses a
         # cached dictionary of the mappings.
-        seq = "N"*68
-        qual = "".join(chr(64+q) for q in range(-5, 63))
+        seq = "N" * 68
+        qual = "".join(chr(64 + q) for q in range(-5, 63))
         expected_phred = [round(QualityIO.phred_quality_from_solexa(q))
                           for q in range(-5, 63)]
         in_handle = StringIO("@Test\n%s\n+\n%s" % (seq, qual))
@@ -684,8 +686,8 @@ class MappingTests(unittest.TestCase):
 
     def test_sanger_to_illumina(self):
         """Mapping check for FASTQ Sanger (0 to 93) to Illumina (0 to 62)"""
-        seq = "N"*94
-        qual = "".join(chr(33+q) for q in range(0, 94))
+        seq = "N" * 94
+        qual = "".join(chr(33 + q) for q in range(0, 94))
         expected_phred = [min(62, q) for q in range(0, 94)]
         in_handle = StringIO("@Test\n%s\n+\n%s" % (seq, qual))
         out_handle = StringIO()
@@ -702,8 +704,8 @@ class MappingTests(unittest.TestCase):
 
     def test_illumina_to_sanger(self):
         """Mapping check for FASTQ Illumina (0 to 62) to Sanger (0 to 62)"""
-        seq = "N"*63
-        qual = "".join(chr(64+q) for q in range(0, 63))
+        seq = "N" * 63
+        qual = "".join(chr(64 + q) for q in range(0, 63))
         expected_phred = list(range(63))
         in_handle = StringIO("@Test\n%s\n+\n%s" % (seq, qual))
         out_handle = StringIO()
