@@ -16,8 +16,8 @@ from Bio import Alphabet
 from Bio.Align import MultipleSeqAlignment
 
 
-#List of formats including alignment only file formats we can read AND write.
-#We don't care about the order
+# List of formats including alignment only file formats we can read AND write.
+# We don't care about the order
 test_write_read_alignment_formats = sorted(SeqIO._FormatToWriter)
 for format in sorted(AlignIO._FormatToWriter):
     if format not in test_write_read_alignment_formats:
@@ -62,11 +62,11 @@ test_records = [
 # Meddle with the annotation too:
 assert test_records[4][1] == "3 DNA seq alignment with CR/LF in name/descr"
 # Add a list of strings,
-test_records[4][0][2].annotations["note"] = ["Note%salso" % os.linesep
-                                    + "\r\nhas\n evil line\rbreaks!", "Wow"]
+test_records[4][0][2].annotations["note"] = ["Note%salso" % os.linesep +
+                                             "\r\nhas\n evil line\rbreaks!", "Wow"]
 # Add a simple string
-test_records[4][0][2].annotations["comment"] = "More%sof" % os.linesep \
-                                          + "\r\nthese\n evil line\rbreaks!"
+test_records[4][0][2].annotations["comment"] = ("More%sof" % os.linesep +
+                                               "\r\nthese\n evil line\rbreaks!")
 # Add a float too:
 test_records[4][0][2].annotations["weight"] = 2.5
 
@@ -78,7 +78,7 @@ class WriterTests(unittest.TestCase):
 
         This has some general expected exceptions hard coded!
         """
-        #TODO - Check the exception messages?
+        # TODO - Check the exception messages?
         lengths = len(set(len(r) for r in records))
         if not records and format in ["stockholm", "phylip", "phylip-relaxed",
                                       "phylip-sequential", "nexus", "clustal",
@@ -107,16 +107,16 @@ class WriterTests(unittest.TestCase):
             handle = StringIO()
         count = SeqIO.write(records, handle, format)
         self.assertEqual(count, len(records))
-        #Now read them back...
+        # Now read them back...
         handle.seek(0)
         new_records = list(SeqIO.parse(handle, format))
         self.assertEqual(len(new_records), len(records))
         for record, new_record in zip(records, new_records):
-            #Using compare_record(record, new_record) is too strict
+            # Using compare_record(record, new_record) is too strict
             if format == "nexus":
-                #The nexus parser will dis-ambiguate repeated record ids.
+                # The nexus parser will dis-ambiguate repeated record ids.
                 self.assertTrue(record.id == new_record.id or
-                                new_record.id.startswith(record.id+".copy"))
+                                new_record.id.startswith(record.id + ".copy"))
             else:
                 self.assertEqual(record.id, new_record.id)
             self.assertEqual(str(record.seq), str(new_record.seq))
@@ -136,21 +136,32 @@ class WriterTests(unittest.TestCase):
             self.assertRaises(err_type, SeqIO.write, records, handle, format)
         handle.close()
 
+    def test_bad_handle(self):
+        handle = os.devnull
+        record = SeqRecord(Seq("CHSMAIKLSSEHNIPSGIANAL", Alphabet.generic_protein), id="Alpha")
+        records = [record]
+        format = "fasta"
+        # These deliberately mix up the handle and record order:
+        self.assertRaises(TypeError, SeqIO.write, handle, record, format)
+        self.assertRaises(TypeError, SeqIO.write, handle, records, format)
+        self.assertEqual(1, SeqIO.write(records, handle, format))
+
+
 for (records, descr, errs) in test_records:
     for format in test_write_read_alignment_formats:
-        #Assume no errors expected...
+        # Assume no errors expected...
         def funct(records, format, descr):
-            f = lambda x : x.check(records, format)
+            f = lambda x: x.check(records, format)
             f.__doc__ = "%s for %s" % (format, descr)
             return f
         setattr(WriterTests,
                 "test_%s_%s" % (format, descr.replace(" ", "_")),
                 funct(records, format, descr))
-        #Replace the method with an error specific one?
+        # Replace the method with an error specific one?
         for err_formats, err_type, err_msg in errs:
             if format in err_formats:
                 def funct_e(records, format, descr, err_type, err_msg):
-                    f = lambda x : x.check_write_fails(records, format,
+                    f = lambda x: x.check_write_fails(records, format,
                                                        err_type, err_msg)
                     f.__doc__ = "%s for %s" % (format, descr)
                     return f
@@ -161,5 +172,5 @@ for (records, descr, errs) in test_records:
         del funct
 
 if __name__ == "__main__":
-    runner = unittest.TextTestRunner(verbosity = 2)
+    runner = unittest.TextTestRunner(verbosity=2)
     unittest.main(testRunner=runner)
